import * as i3 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { Injectable, Component, ChangeDetectionStrategy, Inject, Optional, Input, HostListener, NgModule } from '@angular/core';
import { MAT_LEGACY_FORM_FIELD_DEFAULT_OPTIONS } from '@angular/material/legacy-form-field';
import * as i10 from '@angular/router';
import { RouterModule } from '@angular/router';
import { addAllWidgets } from '@smallstack/all-widgets';
import * as i1 from '@smallstack/axios-api-client';
import { ApplicationsApi, AxiosApiClient, TenantDataApi } from '@smallstack/axios-api-client';
import { BackofficeIcons } from '@smallstack/backoffice-shared';
import { SmallstackCmsModule } from '@smallstack/cms-components';
import * as i4 from '@smallstack/common-components';
import { EnvironmentKeys, TRANSLATION_HELPER, ApplicationLoaderComponent, ConvertToSubjectPipe, RESELLER_API_CLIENT, EnvironmentService, TokenService, FrontendSchemaService } from '@smallstack/common-components';
import { Logger, IOC } from '@smallstack/core-common';
import { addDefaultFormInputComponents } from '@smallstack/form-components';
import { addDefaultPdfFormInputComponents } from '@smallstack/form-components/pdf';
import * as i3$1 from '@smallstack/form-core-components';
import { INPUT_WIDGETS_TRANSLATION_STORE } from '@smallstack/form-core-components';
import * as i7 from '@smallstack/i18n-components';
import { SmallstackI18nModule, TranslationStore } from '@smallstack/i18n-components';
import { loadCommon, loadDevices, loadApplication, loadCrm, loadComponents, loadMailings, loadUser, loadWidgets, loadBackofficeContacts, loadBackofficeMessages, loadTodo } from '@smallstack/language-packs';
import { SmallstackProductModule } from '@smallstack/product-components';
import { SmallstackRxModule } from '@smallstack/rx-components';
import * as i12 from '@smallstack/store-components';
import { LoaderComponent } from '@smallstack/store-components';
import * as i2 from '@smallstack/theme-components';
import { SmallstackThemeModule } from '@smallstack/theme-components';
import { SmallstackTodoModule } from '@smallstack/todo-components';
import * as i6$1 from '@smallstack/type-components';
import { TYPE_USERS, TYPE_CUSTOMERS, TYPE_CONTACTS, AjvFactory } from '@smallstack/typesystem';
import * as i6 from '@smallstack/user-components';
import { SmallstackUserModule, ProjectUserStore } from '@smallstack/user-components';
import * as i8$1 from '@smallstack/widget-core';
import { SmallstackWidgetsModule, EXTENSION_SLOT_COMPONENT_PROVIDER } from '@smallstack/widget-core';
import { Store, StoreState } from '@smallstack/store';
import { BehaviorSubject, combineLatest, startWith, filter, switchMap, shareReplay, map, Subscription, first, tap } from 'rxjs';
import * as i9 from '@angular/material/bottom-sheet';
import * as i5 from '@angular/platform-browser';
import { ApplicationConfigurations } from '@smallstack/application-shared';
import * as i8 from '@smallstack/service-worker-components';
import { NewVersionMessageComponent } from '@smallstack/service-worker-components';
import { objectsEqual, filterNullish } from '@smallstack/utils';

const currentApplication$ = new Store(undefined);
const currentPath$ = new BehaviorSubject("/");
const bootstrapMethod$ = new BehaviorSubject(undefined);

/* eslint-disable @typescript-eslint/member-ordering */
class ApplicationService {
    #application$;
    constructor(userService, routerUtilService) {
        this.userService = userService;
        this.routerUtilService = routerUtilService;
        this.#application$ = new BehaviorSubject(undefined);
        this.application$ = this.#application$.asObservable();
        this.currentContent$ = combineLatest([
            this.#application$,
            currentPath$,
            this.userService.onLogout$.pipe(startWith(false))
        ]).pipe(filter(([application, currentPath]) => !!application && !!currentPath), switchMap(async ([application, currentPath]) => {
            try {
                if (!application)
                    return undefined;
                else {
                    if (!(application.content instanceof Array) || application.content.length === 0)
                        throw new Error("no content set on application");
                    else {
                        const applicationContent = application.content?.find((nav) => nav.path === currentPath);
                        // no content?
                        if (!applicationContent) {
                            Logger.error("ContentRendererComponent", "no content entry found for path " + currentPath);
                            return;
                        }
                        // authentication required?
                        if (applicationContent.authenticationRequired === true) {
                            await this.userService.waitForCurrentUserLoaded();
                            if (!this.userService.getCurrentUserSnapshot()) {
                                Logger.info("ApplicationService", "Route '" +
                                    currentPath +
                                    "' requires authentication, but no token stored. Redirecting to login route: " +
                                    application.configuration[ApplicationConfigurations.LOGIN_ROUTE]);
                                void this.routerUtilService.navigateFullUrl(application.configuration[ApplicationConfigurations.LOGIN_ROUTE], true);
                                return undefined;
                            }
                        }
                        // return content
                        return applicationContent;
                    }
                }
            }
            catch (e) {
                console.error(e);
                throw e;
            }
        }), shareReplay());
        this.currentComponent$ = this.currentContent$.pipe(map((applicationContent) => applicationContent?.content));
    }
    setApplication(application) {
        if (objectsEqual(this.#application$.value, application))
            return;
        this.#application$.next(application);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: ApplicationService, deps: [{ token: i6.UserService }, { token: i4.RouterUtilService }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: ApplicationService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: ApplicationService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: i6.UserService }, { type: i4.RouterUtilService }]; } });

/* eslint-disable @typescript-eslint/member-ordering */
const APPLICATION_LOCAL_STORAGE_KEY = "@smallstack/application";
class ApplicationComponent {
    /** Application Id */
    set id(id) {
        void this.#loadById(id);
    }
    /** Application Domain */
    set domain(domain) {
        void this.#loadByDomain(domain);
    }
    set application(application) {
        if (!objectsEqual(this.currentApplication$.value, application))
            this.currentApplication$.setValue(application);
    }
    #subscription;
    constructor(axiosApiClient, themeService, cdr, location, basePathService, translationHelper, title, dataTypeStore, typeRegistry, iconThemeService, applicationService, environmentService, tokenService, contextService, serviceWorkerUpdateService, bottomSheet) {
        this.axiosApiClient = axiosApiClient;
        this.cdr = cdr;
        this.basePathService = basePathService;
        this.dataTypeStore = dataTypeStore;
        this.typeRegistry = typeRegistry;
        this.applicationService = applicationService;
        this.environmentService = environmentService;
        this.tokenService = tokenService;
        this.contextService = contextService;
        this.serviceWorkerUpdateService = serviceWorkerUpdateService;
        this.bottomSheet = bottomSheet;
        this.currentApplication$ = currentApplication$;
        this.editMode$ = bootstrapMethod$.pipe(map((val) => val?.editMode === true));
        this.currentLayoutComponent$ = combineLatest([
            this.currentApplication$.value$,
            this.applicationService.currentContent$
        ]).pipe(map(([currentApplication, currentContent]) => {
            if (!currentApplication || !currentContent?.layout)
                return undefined;
            const layout = currentApplication.layouts.find((layout) => layout.name === currentContent.layout);
            if (layout)
                return layout.widget;
            return undefined;
        }));
        this.#subscription = new Subscription();
        if (this.serviceWorkerUpdateService)
            this.#subscription.add(this.serviceWorkerUpdateService.newVersionReady$
                .pipe(first((newVersionReady) => newVersionReady === true))
                .subscribe(() => {
                this.bottomSheet.open(NewVersionMessageComponent);
            }));
        // set the current path
        this.#setCurrentPath(location.path());
        location.onUrlChange(() => {
            this.#setCurrentPath(location.path());
        });
        // initialize app via bootstrapMethod$?
        if (bootstrapMethod$.value?.method === "byDomain")
            void this.#loadByDomain(bootstrapMethod$.value.value);
        if (bootstrapMethod$.value?.method === "byId")
            void this.#loadById(bootstrapMethod$.value.value);
        if (bootstrapMethod$.value?.method === "byApplication")
            void this.#loadByApplication(bootstrapMethod$.value.value);
        if (bootstrapMethod$.value?.method === "byOrigin")
            void this.#loadByOrigin(bootstrapMethod$.value.value);
        // manage state
        void currentApplication$.error$.subscribe((error) => {
            if (error)
                this.errorMessage = error.message;
            else
                this.errorMessage = undefined;
        });
        // load and manage app + theme + title
        void currentApplication$.value$
            .pipe(filterNullish(), tap((application) => {
            themeService.$currentTheme.next(application?.theme);
            if (application?.configuration?.iconTheme)
                iconThemeService.theme$.next(application.configuration.iconTheme);
            // set title
            title.setTitle(translationHelper.translate(application.pwaConfiguration?.name));
            // set login type
            // application.configuration.
        }))
            .subscribe();
    }
    ngOnDestroy() {
        this.#subscription.unsubscribe();
    }
    async #loadByApplication(application) {
        if (application) {
            await this.#setApplicationState(application);
            this.cdr.markForCheck();
        }
    }
    async #loadById(id) {
        const response = await this.axiosApiClient.get(ApplicationsApi).getApplicationByAppId({ id });
        if (response?.data) {
            await this.#setApplicationState(response.data);
        }
        this.cdr.markForCheck();
    }
    async #loadByDomain(domain) {
        const response = await this.axiosApiClient.get(ApplicationsApi).getApplicationByDomain({ domain });
        switch (response.status) {
            case 200:
                await this.#setApplicationState(response.data);
                break;
            case 404:
                this.errorMessage =
                    "Für die Domain " +
                        domain +
                        " wurde keine Applikation gefunden. Bitte stellen Sie sicher, dass diese Domain ordnungsgemäß mit Ihrer Applikation verbunden wurde.";
                break;
        }
        this.cdr.markForCheck();
    }
    async #loadByOrigin(apiUrl) {
        currentApplication$.setLoading();
        const axiosApiClient = new AxiosApiClient({
            apiUrl,
            axiosConfig: { validateStatus: () => true }
        });
        // load the app
        const application = await axiosApiClient.get(ApplicationsApi).getApplicationByOrigin();
        if (application.status === 200) {
            await this.#setApplicationState(application.data);
        }
        else {
            await this.dataTypeStore.reset();
            currentApplication$.setError(new Error(application.status + " - " + application.statusText));
        }
    }
    async #setApplicationState(application) {
        if (application) {
            // update context service
            this.contextService.setContext({
                authTenantId: () => {
                    return application?.configuration
                        ? application?.configuration[ApplicationConfigurations.AUTH_TYPE] === "customers"
                            ? application?._meta.tenantId
                            : application?._meta.resellerId
                        : application?._meta.resellerId;
                },
                tenantId: () => application?._meta?.tenantId,
                resellerId: () => application?._meta?.resellerId
            });
            // update axios api client
            this.axiosApiClient.apiClientConfiguration.apiUrl = this.environmentService.get(EnvironmentKeys.API_URL);
            this.axiosApiClient.apiClientConfiguration.authTenantId = application?.configuration
                ? application?.configuration[ApplicationConfigurations.AUTH_TYPE] === "customers"
                    ? application?._meta?.tenantId
                    : application?._meta?.resellerId
                : application?._meta?.resellerId;
            this.axiosApiClient.apiClientConfiguration.tenantId = application._meta?.tenantId;
            this.axiosApiClient.apiClientConfiguration.resellerId = application._meta?.resellerId;
            this.axiosApiClient.apiClientConfiguration.token = () => this.tokenService.getLoginToken();
        }
        await this.dataTypeStore.preload();
        currentApplication$.setValue(application, false);
        this.applicationService.setApplication(application);
        const typeConfigurations = await this.axiosApiClient
            .get(TenantDataApi)
            .getTypeConfigurations()
            .then((res) => res.data.elements);
        if (typeConfigurations instanceof Array) {
            for (const typeConfiguration of typeConfigurations) {
                const applicationUrl = typeConfiguration.urls?.find((url) => url.name === "applicationUrl");
                if (applicationUrl)
                    this.typeRegistry.extendType(typeConfiguration.path, (type) => {
                        type.url = applicationUrl.url;
                        return type;
                    });
            }
        }
        currentApplication$.setState(StoreState.LOADED);
    }
    #setCurrentPath(path) {
        if (!path || path === "")
            path = "/";
        path = this.basePathService.removeBasePath(path);
        if (path.endsWith("/"))
            path = path.substring(0, path.length - 1);
        if (!path.startsWith("/"))
            path = "/" + path;
        if (path.includes("?"))
            path = path.substring(0, path.indexOf("?"));
        if (path !== currentPath$.value)
            currentPath$.next(path);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: ApplicationComponent, deps: [{ token: i1.AxiosApiClient }, { token: i2.ThemeService }, { token: i0.ChangeDetectorRef }, { token: i3.Location }, { token: i4.BasePathService }, { token: TRANSLATION_HELPER }, { token: i5.Title }, { token: i4.DataTypeStore }, { token: i6$1.FrontendTypeRegistry }, { token: i2.IconThemeService }, { token: ApplicationService }, { token: i4.EnvironmentService }, { token: i4.TokenService }, { token: i4.ContextService }, { token: i8.ServiceWorkerUpdateService, optional: true }, { token: i9.MatBottomSheet }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.0.0", type: ApplicationComponent, selector: "smallstack-application", inputs: { id: "id", domain: "domain", application: "application" }, providers: [ApplicationService], ngImport: i0, template: "<smallstack-application-loader>\n  <ng-template>\n    <div *ngIf=\"errorMessage; else applicationTpl\" class=\"error-message flex flex-col justify-center items-center\">\n      <smallstack-icon alias=\"error\"></smallstack-icon>\n      <div>{{ errorMessage }}</div>\n    </div>\n    <ng-template #applicationTpl>\n      <ng-container *ngIf=\"currentApplication$.value as application; else loadingApplication\">\n        <ng-container *ngIf=\"currentLayoutComponent$ | async as currentLayoutComponent; else noLayout\">\n          <smallstack-widget-tree\n            [widget]=\"currentLayoutComponent\"\n            [widgetRendererOptions]=\"{ editMode$: editMode$ | convertToSubject }\"\n          ></smallstack-widget-tree>\n        </ng-container>\n        <ng-template #noLayout>\n          <router-outlet></router-outlet>\n        </ng-template>\n      </ng-container>\n      <ng-template #loadingApplication>\n        <smallstack-loader loadingText=\"Applikation wird geladen\"></smallstack-loader>\n      </ng-template>\n    </ng-template>\n  </ng-template>\n</smallstack-application-loader>\n", styles: [".error-message{height:100vh;width:100vw}\n"], dependencies: [{ kind: "directive", type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i10.RouterOutlet, selector: "router-outlet", inputs: ["name"], outputs: ["activate", "deactivate", "attach", "detach"], exportAs: ["outlet"] }, { kind: "component", type: i8$1.WidgetTreeComponent, selector: "smallstack-widget-tree", inputs: ["widgetRendererOptions", "widget", "context", "globals", "connections", "rootSockets"], outputs: ["connectionsChange", "widgetChange"] }, { kind: "component", type: i2.IconComponent, selector: "smallstack-icon", inputs: ["theme", "size", "name", "alias", "action", "actionSize", "spinning", "actionSpinning", "color", "themeColor", "actionColor", "badge", "badgeColor"], outputs: ["iconError"] }, { kind: "component", type: i4.ApplicationLoaderComponent, selector: "smallstack-application-loader" }, { kind: "component", type: i12.LoaderComponent, selector: "smallstack-loader", inputs: ["loading", "color", "size", "loadingText"] }, { kind: "pipe", type: i3.AsyncPipe, name: "async" }, { kind: "pipe", type: i4.ConvertToSubjectPipe, name: "convertToSubject" }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: ApplicationComponent, decorators: [{
            type: Component,
            args: [{ selector: "smallstack-application", changeDetection: ChangeDetectionStrategy.OnPush, providers: [ApplicationService], template: "<smallstack-application-loader>\n  <ng-template>\n    <div *ngIf=\"errorMessage; else applicationTpl\" class=\"error-message flex flex-col justify-center items-center\">\n      <smallstack-icon alias=\"error\"></smallstack-icon>\n      <div>{{ errorMessage }}</div>\n    </div>\n    <ng-template #applicationTpl>\n      <ng-container *ngIf=\"currentApplication$.value as application; else loadingApplication\">\n        <ng-container *ngIf=\"currentLayoutComponent$ | async as currentLayoutComponent; else noLayout\">\n          <smallstack-widget-tree\n            [widget]=\"currentLayoutComponent\"\n            [widgetRendererOptions]=\"{ editMode$: editMode$ | convertToSubject }\"\n          ></smallstack-widget-tree>\n        </ng-container>\n        <ng-template #noLayout>\n          <router-outlet></router-outlet>\n        </ng-template>\n      </ng-container>\n      <ng-template #loadingApplication>\n        <smallstack-loader loadingText=\"Applikation wird geladen\"></smallstack-loader>\n      </ng-template>\n    </ng-template>\n  </ng-template>\n</smallstack-application-loader>\n", styles: [".error-message{height:100vh;width:100vw}\n"] }]
        }], ctorParameters: function () { return [{ type: i1.AxiosApiClient }, { type: i2.ThemeService }, { type: i0.ChangeDetectorRef }, { type: i3.Location }, { type: i4.BasePathService }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [TRANSLATION_HELPER]
                }] }, { type: i5.Title }, { type: i4.DataTypeStore }, { type: i6$1.FrontendTypeRegistry }, { type: i2.IconThemeService }, { type: ApplicationService }, { type: i4.EnvironmentService }, { type: i4.TokenService }, { type: i4.ContextService }, { type: i8.ServiceWorkerUpdateService, decorators: [{
                    type: Optional
                }] }, { type: i9.MatBottomSheet }]; }, propDecorators: { id: [{
                type: Input
            }], domain: [{
                type: Input
            }], application: [{
                type: Input
            }] } });

class ContentRendererComponent {
    constructor(tokenService, applicationService) {
        this.tokenService = tokenService;
        this.applicationService = applicationService;
        this.currentPath$ = currentPath$;
        this.currentApplication$ = currentApplication$;
        this.currentConnections$ = combineLatest([
            currentApplication$.value$,
            currentPath$,
            this.tokenService.tokenValid$
        ]).pipe(map(([application, currentPath, tokenValid]) => {
            if (!application)
                return undefined;
            else {
                if (!(application.content instanceof Array) || application.content.length === 0)
                    throw new Error("no content set on application");
                else {
                    const applicationContent = application.content?.find((nav) => nav.path === currentPath);
                    if (!applicationContent) {
                        // history.back();
                        Logger.error("ContentRendererComponent", "no content entry found for path " + currentPath);
                    }
                    else if (applicationContent.authenticationRequired === true && !tokenValid)
                        return [];
                    else
                        return applicationContent.socketConnections;
                }
            }
        }));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: ContentRendererComponent, deps: [{ token: i4.TokenService }, { token: ApplicationService }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.0.0", type: ContentRendererComponent, selector: "smallstack-application-content-renderer", ngImport: i0, template: "<ng-container *ngIf=\"currentApplication$.value$ | async as app; else noAppLoaded\">\n  <ng-container *ngIf=\"applicationService.currentComponent$ | async as cmsComponent; else notFound\">\n    <smallstack-widget-tree\n      [widget]=\"cmsComponent\"\n      [connections]=\"currentConnections$ | async\"\n    ></smallstack-widget-tree>\n  </ng-container>\n  <ng-template #notFound>\n    <!-- <div data-test=\"not-found-container\" class=\"error-message flex flex-col justify-center items-center\">\n      <smallstack-icon alias=\"error\"></smallstack-icon>\n      <div>Es wurde keine Seite f\u00FCr den Pfad \"{{ currentPath$ | async }}\" gefunden</div>\n    </div> -->\n  </ng-template>\n</ng-container>\n<ng-template #noAppLoaded> Es wurde keine App gefunden f\u00FCr die aktuelle Domain </ng-template>\n", styles: [""], dependencies: [{ kind: "directive", type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "component", type: i8$1.WidgetTreeComponent, selector: "smallstack-widget-tree", inputs: ["widgetRendererOptions", "widget", "context", "globals", "connections", "rootSockets"], outputs: ["connectionsChange", "widgetChange"] }, { kind: "pipe", type: i3.AsyncPipe, name: "async" }], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: ContentRendererComponent, decorators: [{
            type: Component,
            args: [{ selector: "smallstack-application-content-renderer", changeDetection: ChangeDetectionStrategy.OnPush, template: "<ng-container *ngIf=\"currentApplication$.value$ | async as app; else noAppLoaded\">\n  <ng-container *ngIf=\"applicationService.currentComponent$ | async as cmsComponent; else notFound\">\n    <smallstack-widget-tree\n      [widget]=\"cmsComponent\"\n      [connections]=\"currentConnections$ | async\"\n    ></smallstack-widget-tree>\n  </ng-container>\n  <ng-template #notFound>\n    <!-- <div data-test=\"not-found-container\" class=\"error-message flex flex-col justify-center items-center\">\n      <smallstack-icon alias=\"error\"></smallstack-icon>\n      <div>Es wurde keine Seite f\u00FCr den Pfad \"{{ currentPath$ | async }}\" gefunden</div>\n    </div> -->\n  </ng-template>\n</ng-container>\n<ng-template #noAppLoaded> Es wurde keine App gefunden f\u00FCr die aktuelle Domain </ng-template>\n" }]
        }], ctorParameters: function () { return [{ type: i4.TokenService }, { type: ApplicationService }]; } });

class PwaService {
    constructor() {
        console.log("PWAService active");
    }
    saveInstallPromptEvent(e) {
        e.preventDefault();
        this.savedInstallPromptEvent = e;
        Logger.info("PwaService", "Saved pwa install event for later...");
    }
    promptInstall() {
        if (this.savedInstallPromptEvent)
            this.savedInstallPromptEvent.prompt();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: PwaService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: PwaService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: PwaService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return []; }, propDecorators: { saveInstallPromptEvent: [{
                type: HostListener,
                args: ["window:beforeinstallprompt", ["$event"]]
            }] } });

/* eslint-disable @typescript-eslint/member-ordering */
class SmallstackApplicationModule {
    static bootstrapByApplication(application) {
        bootstrapMethod$.next({ method: "byApplication", value: application });
        return SmallstackApplicationModule;
    }
    static bootstrapByDomain(domain) {
        bootstrapMethod$.next({ method: "byDomain", value: domain });
        return SmallstackApplicationModule;
    }
    static bootstrapById(applicationId) {
        bootstrapMethod$.next({ method: "byId", value: applicationId });
        return {
            ngModule: SmallstackApplicationModule
        };
    }
    static bootstrapByPreviewPath() {
        bootstrapMethod$.next({ method: "byPreviewUrl" });
        return SmallstackApplicationModule;
    }
    static async bootstrapByOrigin(apiUrl) {
        bootstrapMethod$.next({ method: "byOrigin", value: apiUrl });
        return SmallstackApplicationModule;
    }
    constructor(iconThemeService, axiosApiClient, schemaFormInputsRegistry, basePathService, storeRegistry, typeRegistry, userStore, customerStore, routerUtilService, applicationLoaderService, localeService, translationLoader, msalLoginService, tokenService, apiConfigurationService, environmentService, widgetRegistry, userService) {
        // msal configuration
        msalLoginService.watchLoginRedirect();
        // add all lazy loading widgets
        addAllWidgets(widgetRegistry);
        // add global loaders
        applicationLoaderService.addLoader(async () => {
            localeService.availableLocales$.next([{ iso: "de_de" }, { iso: "en_us" }]);
            translationLoader.register([
                loadCommon,
                loadDevices,
                loadApplication,
                loadCrm,
                loadComponents,
                loadMailings,
                loadUser,
                loadWidgets,
                loadBackofficeContacts,
                loadBackofficeMessages,
                loadTodo
            ]);
            await localeService.detectLocale();
        });
        apiConfigurationService.configuration$.next({
            apiUrl: () => environmentService.get(EnvironmentKeys.API_URL)
        });
        // on application load
        if (bootstrapMethod$.value?.method === "byPreviewUrl") {
            const pathNames = window.location.pathname.split("/");
            bootstrapMethod$.next({
                method: "byId",
                value: pathNames[3],
                editMode: window.location.search.includes("editMode=true")
            });
            basePathService.basePath$.next("/apps/preview/" + pathNames[3]);
        }
        // icon theme
        iconThemeService.theme$.next("color");
        iconThemeService.setAliases(BackofficeIcons);
        // form fields
        addDefaultFormInputComponents(schemaFormInputsRegistry);
        addDefaultPdfFormInputComponents(schemaFormInputsRegistry);
        // for web components inside applications
        IOC.register("axiosApiClient", axiosApiClient);
        // some stores
        storeRegistry.registerStore(TYPE_USERS, userStore);
        storeRegistry.registerStore(TYPE_CUSTOMERS, customerStore);
        // fix some types
        typeRegistry.extendType(TYPE_CONTACTS, (type) => {
            type.url = "${basePath}/contacts?openDetailsId=${id}";
            return type;
        });
        // close dialogs on navigate
        routerUtilService.enableDialogAutoClosing();
        // update token prefix
        void currentApplication$.value$.subscribe((applicationDto) => {
            if (applicationDto?.id) {
                tokenService.setLoginTokenPropertyPrefix("@smallstack/applications/" + applicationDto.id + "/");
                userService.monitorToken();
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: SmallstackApplicationModule, deps: [{ token: i2.IconThemeService }, { token: i1.AxiosApiClient }, { token: i3$1.SchemaFormInputsRegistry }, { token: i4.BasePathService }, { token: i4.StoreRegistry }, { token: i6$1.FrontendTypeRegistry }, { token: i6.ProjectUserStore }, { token: i6.CustomerStore }, { token: i4.RouterUtilService }, { token: i4.ApplicationLoaderService }, { token: i7.LocaleService }, { token: i7.TranslationLoader }, { token: i6.MsalLoginService }, { token: i4.TokenService }, { token: i4.ApiConfigurationService }, { token: i4.EnvironmentService }, { token: i8$1.WidgetRegistry }, { token: i6.UserService }], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "16.0.0", ngImport: i0, type: SmallstackApplicationModule, declarations: [ApplicationComponent, ContentRendererComponent], imports: [CommonModule, i10.RouterModule, SmallstackWidgetsModule,
            SmallstackThemeModule,
            SmallstackCmsModule,
            SmallstackI18nModule,
            SmallstackTodoModule,
            SmallstackUserModule,
            SmallstackProductModule,
            SmallstackRxModule,
            ApplicationLoaderComponent,
            ConvertToSubjectPipe,
            LoaderComponent], exports: [ApplicationComponent] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: SmallstackApplicationModule, providers: [
            PwaService,
            {
                provide: ProjectUserStore,
                useFactory: (resellerApiClient) => {
                    return new ProjectUserStore(resellerApiClient, () => currentApplication$.value?._meta.tenantId);
                },
                deps: [RESELLER_API_CLIENT]
            },
            {
                provide: RESELLER_API_CLIENT,
                useFactory: (environmentService, tokenService) => {
                    return new AxiosApiClient({
                        apiUrl: environmentService.get(EnvironmentKeys.API_URL),
                        tenantId: () => currentApplication$.value?._meta?.resellerId,
                        resellerId: () => currentApplication$.value?._meta?.resellerId,
                        token: () => tokenService.getLoginToken()
                    });
                },
                deps: [EnvironmentService, TokenService]
            },
            {
                provide: TRANSLATION_HELPER,
                useExisting: TranslationStore
            },
            { provide: INPUT_WIDGETS_TRANSLATION_STORE, useExisting: TranslationStore },
            {
                provide: AjvFactory,
                useFactory: (schemaService) => {
                    return new AjvFactory(schemaService);
                },
                deps: [FrontendSchemaService]
            },
            {
                provide: MAT_LEGACY_FORM_FIELD_DEFAULT_OPTIONS,
                useValue: { appearance: "outline", floatLabel: "always" }
            },
            EXTENSION_SLOT_COMPONENT_PROVIDER
        ], imports: [CommonModule,
            RouterModule.forChild([
                {
                    path: "",
                    component: ApplicationComponent,
                    children: [
                        {
                            path: "**",
                            component: ContentRendererComponent
                        }
                    ]
                }
            ]),
            SmallstackWidgetsModule,
            SmallstackThemeModule,
            SmallstackCmsModule,
            SmallstackI18nModule,
            SmallstackTodoModule,
            SmallstackUserModule,
            SmallstackProductModule,
            SmallstackRxModule,
            ApplicationLoaderComponent,
            LoaderComponent] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: SmallstackApplicationModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        CommonModule,
                        RouterModule.forChild([
                            {
                                path: "",
                                component: ApplicationComponent,
                                children: [
                                    {
                                        path: "**",
                                        component: ContentRendererComponent
                                    }
                                ]
                            }
                        ]),
                        SmallstackWidgetsModule,
                        SmallstackThemeModule,
                        SmallstackCmsModule,
                        SmallstackI18nModule,
                        SmallstackTodoModule,
                        SmallstackUserModule,
                        SmallstackProductModule,
                        SmallstackRxModule,
                        ApplicationLoaderComponent,
                        ConvertToSubjectPipe,
                        LoaderComponent
                    ],
                    declarations: [ApplicationComponent, ContentRendererComponent],
                    exports: [ApplicationComponent],
                    providers: [
                        PwaService,
                        {
                            provide: ProjectUserStore,
                            useFactory: (resellerApiClient) => {
                                return new ProjectUserStore(resellerApiClient, () => currentApplication$.value?._meta.tenantId);
                            },
                            deps: [RESELLER_API_CLIENT]
                        },
                        {
                            provide: RESELLER_API_CLIENT,
                            useFactory: (environmentService, tokenService) => {
                                return new AxiosApiClient({
                                    apiUrl: environmentService.get(EnvironmentKeys.API_URL),
                                    tenantId: () => currentApplication$.value?._meta?.resellerId,
                                    resellerId: () => currentApplication$.value?._meta?.resellerId,
                                    token: () => tokenService.getLoginToken()
                                });
                            },
                            deps: [EnvironmentService, TokenService]
                        },
                        {
                            provide: TRANSLATION_HELPER,
                            useExisting: TranslationStore
                        },
                        { provide: INPUT_WIDGETS_TRANSLATION_STORE, useExisting: TranslationStore },
                        {
                            provide: AjvFactory,
                            useFactory: (schemaService) => {
                                return new AjvFactory(schemaService);
                            },
                            deps: [FrontendSchemaService]
                        },
                        {
                            provide: MAT_LEGACY_FORM_FIELD_DEFAULT_OPTIONS,
                            useValue: { appearance: "outline", floatLabel: "always" }
                        },
                        EXTENSION_SLOT_COMPONENT_PROVIDER
                    ]
                }]
        }], ctorParameters: function () { return [{ type: i2.IconThemeService }, { type: i1.AxiosApiClient }, { type: i3$1.SchemaFormInputsRegistry }, { type: i4.BasePathService }, { type: i4.StoreRegistry }, { type: i6$1.FrontendTypeRegistry }, { type: i6.ProjectUserStore }, { type: i6.CustomerStore }, { type: i4.RouterUtilService }, { type: i4.ApplicationLoaderService }, { type: i7.LocaleService }, { type: i7.TranslationLoader }, { type: i6.MsalLoginService }, { type: i4.TokenService }, { type: i4.ApiConfigurationService }, { type: i4.EnvironmentService }, { type: i8$1.WidgetRegistry }, { type: i6.UserService }]; } });

/**
 * Generated bundle index. Do not edit.
 */

export { APPLICATION_LOCAL_STORAGE_KEY, ApplicationComponent, SmallstackApplicationModule, bootstrapMethod$, currentApplication$, currentPath$ };
//# sourceMappingURL=smallstack-application-components.mjs.map
